from collections.abc import Callable


def is_vantage6_algorithm_func(func: Callable) -> bool:
    """
    Check if the function is decorated with a vantage6 decorator, which all
    functions being called in vantage6 algorithm should be

    Parameters
    ----------
    func : Callable
        The function to check

    Returns
    -------
    bool
        True if the function is decorated with a vantage6 decorator, False otherwise
    """
    return get_vantage6_decorator_type(func) is not None


def get_vantage6_decorator_type(func: Callable) -> str | None:
    """
    Get the vantage6 decorator type of the function

    Parameters
    ----------
    func : Callable
        The function to get the vantage6 decorator type of

    Returns
    -------
    str | None
        The vantage6 decorator type of the function, or None if the function is not
        decorated with a vantage6 decorator
    """
    return getattr(func, "vantage6_decorator_step_type", None)



"""
This module provides a client interface for the node to communicate with the
central server.
"""

import datetime
import os
import time
from threading import Thread

from keycloak import KeycloakOpenID

from vantage6.common import WhoAmI
from vantage6.common.client.client_base import ClientBase
from vantage6.common.globals import (
    InstanceType,
    RequiredNodeEnvVars,
)


class NodeClient(ClientBase):
    """Node interface to the central server."""

    def __init__(
        self, node_account_name: str, api_key: str, server_url: str, auth_url: str
    ):
        """
        Initialize the node client.

        Parameters
        ----------
        node_account_name : str
            The name of the node account.
        api_key : str
            The api key of the node.
        server_url : str
            The url of the server to connect to.
        auth_url : str
            The url of the authentication server.
        """
        super().__init__(server_url=server_url, auth_url=auth_url)

        self.node_account_name = node_account_name
        self.api_key = api_key

        # self.name = None
        self.collaboration_id = None
        self.whoami = None
        self.keycloak_client_id = None

        self.run = self.Run(self)
        self.algorithm_store = self.AlgorithmStore(self)
        self.column = self.Column(self)

        self.kc_openid = KeycloakOpenID(
            server_url=self.auth_url,
            realm_name=os.environ.get(RequiredNodeEnvVars.KEYCLOAK_REALM.value),
            client_id=f"{self.node_account_name}-node-client",
            client_secret_key=self.api_key,
        )
        self.token_validity_period = None

    def authenticate(self) -> None:
        """
        Nodes authentication by using keycloak.
        """
        # get token from keycloak
        try:
            self.obtain_new_token()
        except Exception as e:
            self.log.exception("Getting token failed: %s", e)
            raise e

        # get info on how the server sees this node
        node = self.request("node/me")

        name = node.get("name")
        self.collaboration_id = node.get("collaboration").get("id")

        organization_id = node.get("organization").get("id")
        organization = self.request(f"organization/{organization_id}")
        organization_name = organization.get("name")

        self.whoami = WhoAmI(
            type_=InstanceType.NODE.value,
            id_=node["id"],
            name=name,
            organization_id=organization_id,
            organization_name=organization_name,
        )

    def obtain_new_token(self) -> None:
        """Get a new token."""
        self.log.debug("Obtaining new token")
        token = self.kc_openid.token(grant_type="client_credentials")
        self._access_token = token["access_token"]
        decoded_token = self.kc_openid.decode_token(self._access_token)
        self.token_validity_period = decoded_token["exp"] - decoded_token["iat"]

    def auto_renew_token(self) -> None:
        """Start a thread that refreshes token before it expires."""
        # set up thread to refresh token
        t = Thread(target=self.__renew_token_worker, daemon=True)
        t.start()

    def __renew_token_worker(self) -> None:
        """Keep refreshing token to prevent it from expiring."""
        # set variable to start trying to refresh the token 3/4 of the time before
        # it expires
        period_start_refresh = self.token_validity_period / 4
        # set variable to sleep between refreshes - so that we have about 20 attempts
        # before the token expires
        interval_between_refreshes = period_start_refresh / 20
        token_expired = False
        while True:
            # get the time until the token expires
            expiry_time = self.kc_openid.decode_token(self.token)["exp"]
            time_until_expiry = expiry_time - time.time()
            if time_until_expiry < period_start_refresh:
                if time_until_expiry < 0 and not token_expired:
                    token_expired = True
                    self.log.error("Token has expired. Some requests may have failed!")
                    # if token expires, keycloak service is down, or internet connection
                    # of the node is down. We keep on trying to refresh the token but
                    # not too often.
                    interval_between_refreshes = 60  # 1 minute
                try:
                    self.obtain_new_token()
                    token_expired = False
                except Exception as e:
                    self.log.error("Getting new token failed: %s", e)
                    # sleep for a bit and then try again. The server might be
                    # unreachable or internet connection down. We sleep so long that
                    # we should have about 20 attempts before the token expires.
                    time.sleep(interval_between_refreshes)
            else:
                time.sleep(int(time_until_expiry - period_start_refresh + 1))

    def request_token_for_container(self, task_id: int, image: str) -> dict:
        """Request a container-token at the central server.

        This token is used by algorithm containers that run on this
        node. These algorithms can then create subtasks and retrieve
        subresults.

        The server performs a few checks (e.g. if the task you
        request the key for is in progress) before handing out this
        token.

        Parameters
        ----------
        task_id : int
            id from the task, which is going to use this container token
        image : str
            Docker image name of the task

        Returns
        -------
        dict
            The container token.
        """
        self.log.debug(
            "Requesting container token for task_id=%s and image=%s", task_id, image
        )
        return self.request(
            "/token/container", method="post", json={"task_id": task_id, "image": image}
        )

    class Run(ClientBase.SubClient):
        """Subclient for the run endpoint."""

        def list(
            self, state: str, include_task: bool, task_id: int = None
        ) -> dict | list:
            """
            Obtain algorithm runs.

            Parameters
            ----------
            state : str
                State of the desired algorithm runs.
            include_task : bool, optional
                Include the task
            task_id : int, optional
                ID of the task, by default None. If None, all tasks are
                returned.

            Returns
            -------
            dict | list
                The algorithm runs as json.
            """
            params = {"state": state, "node_id": self.parent.whoami.id_}
            if include_task:
                params["include"] = "task"
            if task_id:
                params["task_id"] = task_id
            run_data = self.parent.request(endpoint="run", params=params)

            if isinstance(run_data, str):
                self.parent.log.warning("Requesting algorithm runs failed")
                self.parent.log.warning(f"Fail message: {run_data}")
                return {}

            # if there are multiple pages of algorithm runs, get them all
            links = run_data.get("links")

            page = 1
            while links and links.get("next"):
                page += 1
                next_page = self.parent.request(
                    endpoint="run", params={**params, "page": page}
                )
                run_data["data"] += next_page["data"]
                links = next_page.get("links")

            # strip pagination links
            run_data = run_data["data"]

            # Multiple runs
            for run in run_data:
                run["arguments"] = self.parent._fetch_and_decrypt_run_data(
                    run["arguments"], run["blob_storage_used"]
                )

            return run_data

        def patch(self, id_: int, data: dict, init_org_id: int = None) -> dict | None:
            """
            Update the algorithm run data at the central server.

            Typically used for task status updates (started, finished, etc)

            Parameters
            ----------
            id_: int
                ID of the run to patch
            data: Dict
                Dictionary of fields that are to be patched
            init_org_id: int, optional
                Organization id of the origin of the task. This is required
                when the run dict includes results, because then results have
                to be encrypted specifically for them

            Returns
            -------
            dict | None
                The response from the server, or None if wrong data was provided
            """
            if "result" in data:
                if not init_org_id:
                    self.parent.log.critical(
                        "Organization id is not provided: cannot send results "
                        "to server as they cannot be encrypted"
                    )
                    return
                self.parent.log.debug(
                    f"Retrieving public key from organization={init_org_id}"
                )

                org = self.parent.request(f"organization/{init_org_id}")
                public_key = None
                try:
                    public_key = org["public_key"]
                except KeyError:
                    self.parent.log.critical(
                        "Public key could not be retrieved... Does the "
                        "initiating organization belong to your organization?"
                    )

                self.parent.log.debug("Sending algorithm run update to server")
                blob_store_enabled = self.parent.check_if_blob_store_enabled()
                # If the result is a blob, it is not base64 encoded.
                data["result"] = self.parent.cryptor.encrypt_bytes_to_str(
                    data["result"],
                    public_key,
                    skip_base64_encoding_of_msg=blob_store_enabled,
                )
                # If blob store is enabled, stream the result to the server, which
                # will stream it to the blob store and return the UUID reference to use.
                if blob_store_enabled:
                    result_uuid = self.parent._upload_run_data_to_server(data["result"])
                    self.parent.log.debug(
                        f"Result uploaded to server with UUID: {result_uuid}"
                    )
                    data["result"] = result_uuid
            return self.parent.request(f"run/{id_}", json=data, method="patch")

    class AlgorithmStore(ClientBase.SubClient):
        """Subclient for the algorithm store endpoint."""

        def get(self, id_) -> dict:
            """
            Obtain algorithm store from the central server.

            Parameters
            ----------
            id_ : int
                ID of the algorithm.

            Returns
            -------
            dict
                The algorithms as json.
            """
            return self.parent.request(f"algorithmstore/{id_}")

    class Column(ClientBase.SubClient):
        """Subclient for the column endpoint."""

        def post(self, df_id: int, data: dict) -> dict:
            """
            Create a column at the central server.

            Parameters
            ----------
            df_id: int
                ID of the dataframe.
            data: dict
                Dictionary of fields that are to be patched

            Returns
            -------
            dict
                The response from the server.
            """
            return self.parent.request(
                f"session/dataframe/{df_id}/column", method="post", json=data
            )

    def is_encrypted_collaboration(self) -> bool:
        """
        Check whether the encryption is enabled.

        End-to-end encryption is per collaboration managed at the
        central server. It is important to note that the local
        configuration-file should allow explicitly for unencrpyted
        messages. This function returns the setting from the server.

        Returns
        -------
        bool
            True if the collaboration is encrypted, False otherwise.
        """
        response = self.request(f"collaboration/{self.collaboration_id}")
        return response.get("encrypted") == 1

    def set_run_start_time(self, id_: int) -> None:
        """
        Sets the start time of the task at the central server.

        This is important as this will note that the task has been
        started, and is waiting for restuls.

        Parameters
        ----------
        id_ : int
            ID of the task.
        """
        self.run.patch(
            id_,
            data={
                "started_at": datetime.datetime.now(datetime.timezone.utc).isoformat()
            },
        )

    def check_user_allowed_to_send_task(
        self,
        allowed_users: list[str],
        allowed_orgs: list[str],
        init_org_id: int,
        init_user_id: int,
    ) -> bool:
        """
        Check if the user is allowed to send a task to this node

        Parameters
        ----------
        allowed_users: list[str]
            List of allowed user IDs or usernames
        allowed_orgs: list[str]
            List of allowed organization IDs or names
        init_org_id: int
            ID of the organization that initiated the task
        init_user_id: int
            ID of the user that initiated the task

        Returns
        -------
        bool
            Whether or not the user is allowed to send a task to this node
        """
        # check if task-initating user id is in allowed users
        if any(str(init_user_id) == user for user in allowed_users):
            return True

        # check if task-initiating org id is in allowed orgs
        if any(str(init_org_id) == org for org in allowed_orgs):
            return True

        # TODO it would be nicer to check all users in a single request
        # but that requires other multi-filter options in the API
        # TODO this option is now disabled since nodes do not have permission
        # to access user information. We need to decide if we want to give them
        # that permission for this.
        # ----------------------------------------------------------
        # check if task-initiating user name is in allowed users
        # for user in allowed_users:
        #     resp = self.request("user", params={"username": user})
        #     print(resp)
        #     for d in resp:
        #         if d.get("username") == user and d.get("id") == init_user_id:
        #             return True

        # check if task-initiating org name is in allowed orgs
        for allowed_org in allowed_orgs:
            resp = self.request("organization", params={"name": allowed_org})
            for org in resp:
                if org.get("name") == allowed_org and org.get("id") == init_org_id:
                    return True

        # not in any of the allowed users or orgs
        return False



from dataclasses import dataclass
from typing import Self

from vantage6.common.enum import DatabaseType


@dataclass
class TaskDB:
    """
    Dataclass for a task database.

    This class is used to represent a database from a node configuration file - i.e. it
    is (mainly) used by the node and CLI, and not for user-facing code.

    Attributes
    ----------
    label: str
        The label of the database
    type: DatabaseType
        The type of the database
    uri: str
        The uri of the database
    """

    label: str
    type: DatabaseType
    uri: str
    is_file: bool | None = None
    is_dir: bool | None = None
    env: dict | None = None
    local_uri: str | None = None

    @classmethod
    def from_dict(cls, data: dict) -> Self:
        if "label" not in data:
            raise ValueError("Database label is required")
        if "type" not in data:
            raise ValueError(
                f"Database with label {data['label']} is missing a type. "
                "Please provide a valid type."
            )
        if data["type"] not in DatabaseType.list():
            raise ValueError(
                f"Database with label {data['label']} has an invalid type: "
                f"{data['type']}. Please provide a valid type."
            )
        if "uri" not in data:
            raise ValueError(
                f"Database with label {data['label']} has no uri. "
                "Please provide a valid uri."
            )
        return cls(
            label=data["label"],
            type=DatabaseType(data["type"]),
            uri=data["uri"],
        )



# TODO v5+ remove this file - just a dummy to prevent import issues from v4 CLI for
# the time being


class NetworkManager:
    def __init__(self):
        pass

    def create_network(self, name: str) -> None:
        pass

    def remove_network(self, name: str) -> None:
        pass



from __future__ import annotations

import collections
from abc import abstractmethod
from pathlib import Path
from typing import Any, Type

import yaml
from jinja2 import Environment, FileSystemLoader, Template
from schema import Schema, SchemaError

from vantage6.common.globals import SANDBOX_SUFFIX

from vantage6.cli.globals import TEMPLATE_FOLDER


class Configuration(collections.UserDict):
    """Base class to contain a single configuration."""

    VALIDATORS = {}

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def __setitem__(self, key: str, value: Any) -> None:
        """
        Validation of a single item when put

        Parameters
        ----------
        key: str
            The key of the item to set.
        value: Any
            The value of the item to set.

        Raises
        ------
        AssertionError
            If the value is not valid.
        """
        # assert key in self.VALIDATORS.keys(), "Invalid Key!"
        schema = Schema(
            self.VALIDATORS.get(key, lambda x: True), ignore_extra_keys=True
        )
        assert schema.is_valid(value), (
            f"Invalid value '{value}' provided for field '{key}'"
        )
        super().__setitem__(key, value)

    def __getitem__(self, key: str) -> Any:
        """
        Get an item from the configuration.

        Parameters
        ----------
        key: str
            The key of the item to get.

        Returns
        -------
        Any
            The value of the item.

        Raises
        ------
        KeyError
            If the key is not in the configuration.
        """
        if key in self.data:
            return super().__getitem__(key)
        else:
            raise KeyError(key)

    @property
    def is_valid(self) -> bool:
        """
        Check if the configuration is valid.

        Returns
        -------
        bool
            Whether or not the configuration is valid.
        """
        schema = Schema(self.VALIDATORS, ignore_extra_keys=True)
        is_valid = schema.is_valid(self.data)
        if not is_valid:
            try:
                schema.validate(self.data)
            except SchemaError as exc:
                raise SchemaError(f"Invalid configuration: {exc}") from exc
        return is_valid


class ConfigurationManager(object):
    """Class to maintain valid configuration settings.

    Parameters
    ----------
    name: str
        The name of the configuration.
    conf_class: Configuration
        The class to use for the configuration.
    """

    def __init__(
        self,
        name: str,
        conf_class: Configuration = Configuration,
        is_sandbox: bool = False,
    ) -> None:
        self.config = {}

        self.name = name
        self.is_sandbox = is_sandbox
        self.conf_class = conf_class

    def put(self, config: dict) -> None:
        """
        Add a configuration to the configuration manager.

        Parameters
        ----------
        config: dict
            The configuration to add.

        Raises
        ------
        AssertionError
            If the configuration is not valid.
        """
        configuration = self.conf_class(config)
        if configuration.is_valid:
            self.config = config

    def get(self) -> Configuration:
        """
        Get a configuration from the configuration manager.

        Returns
        -------
        Configuration
            The configuration.
        """
        return self.config

    @property
    def is_empty(self) -> bool:
        """
        Check if the configuration manager is empty.

        Returns
        -------
        bool
            Whether or not the configuration manager is empty.
        """
        return not self.config

    def load(self, path: Path | str) -> None:
        """
        Load a configuration from a file.

        Parameters
        ----------
        path: Path | str
            The path to the file to load the configuration from.
        """
        with open(str(path), "r") as f:
            config = yaml.safe_load(f)

        self.put(config)

    @classmethod
    def from_file(
        cls,
        path: Path | str,
        conf_class: Type[Configuration] = Configuration,
        is_sandbox: bool = False,
    ) -> ConfigurationManager:
        """
        Load a configuration from a file.

        Parameters
        ----------
        path: Path | str
            The path to the file to load the configuration from.
        conf_class: Type[Configuration]
            The class to use for the configuration.
        is_sandbox: bool
            Whether the configuration is a sandbox configuration, by default False
        Returns
        -------
        ConfigurationManager
            The configuration manager with the configuration.

        Raises
        ------
        AssertionError
            If the name of the configuration could not be extracted from the
            file path.
        """
        name = Path(path).stem
        # We do not want the suffix .sandbox in the name, we capture this property in
        # the is_sandbox property of the Context class.
        if is_sandbox:
            name = name.replace(SANDBOX_SUFFIX, "")
        assert name, f"Configuration name could not be extracted from filepath={path}"
        conf = cls(name=name, conf_class=conf_class, is_sandbox=is_sandbox)
        conf.load(path)
        return conf

    def save(self, path: Path | str) -> Path:
        """
        Save the configuration to a file.

        Parameters
        ----------
        path: Path | str
            The path to the file to save the configuration to.

        Returns
        -------
        Path
            The path to the saved configuration file.
        """
        Path(path).parent.mkdir(parents=True, exist_ok=True)
        template = self.get_config_template()
        rendered_config = template.render(self.config)
        # add .sandbox to the path if it is a sandbox configuration
        if self.is_sandbox:
            name = Path(path).stem + SANDBOX_SUFFIX
            path = Path(path).parent / (name + str(Path(path).suffix))

        with open(path, "w", encoding="utf-8") as f:
            f.write(rendered_config)

        return Path(path)

    @abstractmethod
    def get_config_template(self) -> Template:
        """
        Get the configuration template for the server.
        """
        pass

    @classmethod
    def _get_config_template(cls, template_file: str) -> Template:
        """
        Get the configuration template for the server.

        Parameters
        ----------
        template_file: str
            The name of the template file to get.

        Returns
        -------
        jinja2.Template
            The configuration template.
        """
        environment = Environment(
            loader=FileSystemLoader(TEMPLATE_FOLDER),
            trim_blocks=True,
            lstrip_blocks=True,
            autoescape=True,
        )
        return environment.get_template(template_file)



import enum
import logging
import logging.handlers
import os
import sys
from abc import abstractmethod
from pathlib import Path
from typing import Self, Tuple

import appdirs
import pyfiglet

from vantage6.common import Fore, Singleton, Style, __version__, error, get_config_path
from vantage6.common.colors import ColorStreamHandler
from vantage6.common.configuration_manager import ConfigurationManager
from vantage6.common.docker.addons import running_in_docker
from vantage6.common.globals import APPNAME, SANDBOX_SUFFIX, InstanceType


class AppContext(metaclass=Singleton):
    """
    Base class from which to create Node and Server context classes.
    """

    # FIXME: drop the prefix "INST_": a *class* is assigned.
    # FIXME: this does not need to be a class attribute, but ~~can~~_should_
    #        be set in __init__
    #        I think the same applies to LOGGING_ENABLED
    INST_CONFIG_MANAGER = ConfigurationManager
    LOGGING_ENABLED = True
    LOGGER_PREFIX = ""

    def __init__(
        self,
        instance_type: InstanceType,
        instance_name: str,
        system_folders: bool = False,
        config_file: Path | str = None,
        print_log_header: bool = True,
        logger_prefix: str = "",
        in_container: bool = False,
        is_sandbox: bool = False,
    ) -> None:
        """
        Create a new AppContext instance.

        Parameters
        ----------
        instance_type: InstanceType
            Type of instance that is initialized
        instance_name: str
            Name of the configuration
        system_folders: bool
            Use system folders instead of user folders
        config_file: str
            Path to a specific config file. If left as None, OS specific folder
            will be used to find the configuration file specified by
            `instance_name`.
        print_log_header: bool
            Print a banner to the log file.
        in_container: bool
            Whether the application is running inside a container, by default False
        is_sandbox: bool
            Whether the configuration is a sandbox configuration, by default False
        """
        self.LOGGER_PREFIX = logger_prefix
        self.initialize(
            instance_type,
            instance_name,
            system_folders,
            config_file,
            print_log_header,
            in_container,
            is_sandbox,
        )

    def initialize(
        self,
        instance_type: InstanceType,
        instance_name: str,
        system_folders: bool = False,
        config_file: str | None = None,
        print_log_header: bool = True,
        in_container: bool = False,
        is_sandbox: bool = False,
    ) -> None:
        """
        Initialize the AppContext instance.

        Parameters
        ----------
        instance_type: InstanceType
            InstanceType enum
        instance_name: str
            Name of the configuration
        system_folders: bool
            Use system folders rather than user folders
        config_file: str
            Path to a specific config file. If left as None, OS specific folder
            will be used to find the configuration file specified by
            `instance_name`.
        print_log_header: bool
            Print a banner to the log file.
        in_container: bool
            Whether the application is running inside a container, by default False
        is_sandbox: bool
            Whether the configuration is a sandbox configuration, by default False
        """
        self.scope: str = "system" if system_folders else "user"
        self.name: str = instance_name
        if is_sandbox:
            self.name = self.name.replace(SANDBOX_SUFFIX, "")
        self.instance_type: InstanceType = instance_type
        self.in_container = in_container
        self.config_file = self.find_config_file(
            instance_type, self.name, system_folders, config_file, is_sandbox=is_sandbox
        )
        self.is_sandbox = is_sandbox

        # look up system / user directories. This way we can check if the
        # config file container custom directories
        self.set_folders(instance_type, self.name, system_folders)

        # after the folders have been set, we can start logging!
        if self.in_container and self.LOGGING_ENABLED:
            self.setup_logging()

        # Log some history
        # FIXME: this should probably be moved to the actual app
        module_name = __name__.split(".")[-1]
        self.log = logging.getLogger(module_name)
        if print_log_header:
            self.print_log_header()

    def print_log_header(self) -> None:
        """
        Print the log file header.
        """
        self.log.info("-" * 45)
        # self.log.info(f'#{APPNAME:^78}#')
        self.log.info(" Welcome to")
        for line in pyfiglet.figlet_format(APPNAME, font="big").split("\n"):
            self.log.info(line)
        self.log.info(" --> Join us on Discord! https://discord.gg/rwRvwyK")
        self.log.info(" --> Docs: https://docs.vantage6.ai")
        self.log.info(" --> Blog: https://vantage6.ai")
        self.log.info("-" * 60)
        self.log.info("Cite us!")
        self.log.info("If you publish your findings obtained using vantage6, ")
        self.log.info("please cite the proper sources as mentioned in:")
        self.log.info("https://vantage6.ai/vantage6/references")
        self.log.info("-" * 60)
        self.log.info("Started application %s", APPNAME)
        self.log.info("Current working directory is '%s'", os.getcwd())
        self.log.info("Successfully loaded configuration from '%s'", self.config_file)
        self.log.info("Logging to '%s'", self.log_file)
        self.log.info("Common package version '%s'", __version__)

    @classmethod
    def from_external_config_file(
        cls,
        path: Path | str,
        instance_type: InstanceType,
        system_folders: bool = False,
        in_container: bool = False,
        is_sandbox: bool = False,
    ) -> Self:
        """
        Create a new AppContext instance from an external config file.

        Parameters
        ----------
        path: str
            Path to the config file
        instance_type: InstanceType
            Type of instance for which the config file is used
        system_folders: bool
            Use system folders rather than user folders
        in_container: bool
            Whether the application is running inside a container, by default False
        is_sandbox: bool
            Whether the configuration is a sandbox configuration, by default False
        Returns
        -------
        AppContext
            A new AppContext instance
        """
        instance_name = Path(path).stem
        if is_sandbox:
            instance_name = instance_name.replace(SANDBOX_SUFFIX, "")

        self_ = cls.__new__(cls)
        self_.initialize(
            instance_type,
            instance_name,
            system_folders,
            path,
            in_container=in_container,
            is_sandbox=is_sandbox,
        )

        return self_

    @classmethod
    @abstractmethod
    def config_exists(
        cls,
        instance_name: str,
        system_folders: bool,
        is_sandbox: bool,
    ) -> None:
        """Remove the config file if it exists."""

    @classmethod
    def base_config_exists(
        cls,
        instance_type: InstanceType,
        instance_name: str,
        system_folders: bool = False,
        is_sandbox: bool = False,
    ) -> bool:
        """Check if a config file exists for the given instance type and name.

        Parameters
        ----------
        instance_type: InstanceType
            Type of instance that is checked
        instance_name: str
            Name of the configuration
        system_folders: bool
            Use system folders rather than user folders
        is_sandbox: bool
            Whether the configuration is a sandbox configuration, by default False
        Returns
        -------
        bool
            True if the config file exists, False otherwise
        """
        try:
            config_file = cls.find_config_file(
                instance_type,
                instance_name,
                system_folders,
                verbose=False,
                is_sandbox=is_sandbox,
            )

        except FileNotFoundError:
            return False

        # check that configuration is present in config-file
        config = cls.INST_CONFIG_MANAGER.from_file(config_file, is_sandbox=is_sandbox)
        return bool(config)

    @classmethod
    def remove_config_file_if_exists(
        cls,
        instance_type: InstanceType,
        instance_name: str,
        system_folders: bool,
        is_sandbox: bool,
    ) -> None:
        """
        Remove the config file if it exists.

        Parameters
        ----------
        instance_type: InstanceType
            Type of instance that is checked
        instance_name: str
            Name of the configuration
        system_folders: bool
            Use system folders rather than user folders
        is_sandbox: bool
            Whether the configuration is a sandbox configuration, by default False
        """
        if cls.config_exists(instance_name, system_folders, is_sandbox):
            os.remove(
                cls.find_config_file(
                    instance_type, instance_name, system_folders, is_sandbox=is_sandbox
                )
            )

    @staticmethod
    def type_data_folder(instance_type: InstanceType, system_folders: bool) -> Path:
        """
        Return OS specific data folder.

        Parameters
        ----------
        instance_type: InstanceType
            Type of instance that is checked
        system_folders: bool
            Use system folders rather than user folders

        Returns
        -------
        Path
            Path to the data folder
        """
        d = appdirs.AppDirs(APPNAME, "")

        if system_folders:
            return Path(d.site_data_dir) / instance_type

        else:
            return Path(d.user_data_dir) / instance_type

    @staticmethod
    def instance_folders(
        instance_type: InstanceType, instance_name: str, system_folders: bool
    ) -> dict:
        """
        Return OS and instance specific folders for storing logs, data and
        config files.

        Parameters
        ----------
        instance_type: InstanceType
            Type of instance that is checked
        instance_name: str
            Name of the configuration
        system_folders: bool
            Use system folders rather than user folders

        Returns
        -------
        dict
            Dictionary with Paths to the folders of the log, data and config
            files.
        """
        d = appdirs.AppDirs(APPNAME, "")

        if isinstance(instance_type, enum.Enum):
            instance_type = instance_type.value

        if running_in_docker():
            return {
                "log": Path("/mnt/log"),
                "data": Path("/mnt/data"),
                "config": Path("/mnt/config"),
            }
        elif system_folders:
            return {
                "log": Path(d.site_data_dir) / instance_type / instance_name,
                "data": Path(d.site_data_dir) / instance_type / instance_name,
                "config": (Path(get_config_path(d, system_folders)) / instance_type),
            }
        else:
            return {
                "log": Path(d.user_log_dir) / instance_type / instance_name,
                "data": Path(d.user_data_dir) / instance_type / instance_name,
                "config": Path(d.user_config_dir) / instance_type,
                "dev": Path(d.user_config_dir) / "dev",
            }

    @classmethod
    def available_configurations(
        cls, instance_type: InstanceType, system_folders: bool, is_sandbox: bool = False
    ) -> tuple[list[ConfigurationManager], list[Path]]:
        """
        Returns a list of configuration managers and a list of paths to
        configuration files that could not be loaded.

        Parameters
        ----------
        instance_type: InstanceType
            Type of instance that is checked
        system_folders: bool
            Use system folders rather than user folders
        is_sandbox: bool
            When True, instead of showing the regular configurations, show the sandbox
            configurations.

        Returns
        -------
        list[ConfigurationManager], list[Path]
            A list of configuration managers and a list of paths to
            configuration files that could not be loaded.
        """
        folders = cls.instance_folders(instance_type, "", system_folders)
        # potential configuration files
        config_files = Path(folders["config"]).glob(
            "*.yaml" if not is_sandbox else "*.sandbox.yaml"
        )
        configs = []
        failed = []
        for file_ in config_files:
            try:
                conf_manager = cls.INST_CONFIG_MANAGER.from_file(
                    file_, is_sandbox=is_sandbox
                )
                if conf_manager.is_empty:
                    failed.append(file_)
                else:
                    configs.append(conf_manager)
            except Exception as e:
                print(f"Error loading configuration file: {e}")

                failed.append(file_)

        return configs, failed

    @property
    def log_file(self) -> Path:
        """Return the path to the log file.

        Returns
        -------
        Path
            Path to the log file
        """
        return self.log_file_name(type_=self.instance_type.value)

    def log_file_name(self, type_: str) -> Path:
        """
        Return a path to a log file for a given log file type

        Parameters
        ----------
        type_: str
            The type of log file to return.

        Returns
        -------
        Path
            The path to the log file.

        Raises
        ------
        AssertionError
            If the configuration manager is not initialized.
        """
        assert self.config_manager, "Log file unkown. Initialize configuration manager"

        file_ = f"{type_}_{self.scope}.log"
        return self.log_dir / file_

    @property
    def helm_release_name(self) -> str:
        """
        Get the name of the Helm release for the server.
        """
        return f"{APPNAME}-{self.name}-{self.scope}-{self.instance_type.value}"

    @property
    def config_file_name(self) -> str:
        """Return the name of the configuration file.

        Returns
        -------
        str
            Name of the configuration file
        """
        return self.__config_file.stem

    @property
    def config_file(self) -> Path:
        """Return the path to the configuration file.

        Returns
        -------
        Path
            Path to the configuration file
        """
        return self.__config_file

    @config_file.setter
    def config_file(self, path: str) -> None:
        """
        Set the path to the configuration file.

        Parameters
        ----------
        path: str
            Path to the configuration file

        Raises
        ------
        AssertionError
            If the configuration file does not exist
        """
        assert Path(path).exists(), f"config {path} not found"
        self.__config_file = Path(path)
        self.config_manager = self.INST_CONFIG_MANAGER.from_file(path)
        self.config = self.config_manager.config

    @classmethod
    def find_config_file(
        cls,
        instance_type: InstanceType,
        instance_name: str,
        system_folders: bool,
        config_file: str | None = None,
        verbose: bool = True,
        is_sandbox: bool = False,
    ) -> str:
        """
        Find a configuration file.

        Parameters
        ----------
        instance_type: InstanceType
            Type of instance that is checked
        instance_name: str
            Name of the configuration
        system_folders: bool
            Use system folders rather than user folders
        config_file: str | None
            Name of the configuration file. If None, the name of the
            configuration is used.
        verbose: bool
            Print the directories that are searched for the configuration file.
        is_sandbox: bool
            Whether the configuration is a sandbox configuration, by default False
        Returns
        -------
        str
            Path to the configuration file

        Raises
        ------
        FileNotFoundError
            If the configuration file is not found
        """
        if config_file is None:
            config_file = (
                f"{instance_name}.yaml"
                if not is_sandbox
                else f"{instance_name}.sandbox.yaml"
            )

        config_dir = cls.instance_folders(
            instance_type, instance_name, system_folders
        ).get("config")

        dirs = [
            config_dir,
            "./",
        ]

        for location in dirs:
            # If config_file is an absolute path `os.path.join()` ignores
            # `location`.
            fullpath = os.path.join(location, config_file)

            if os.path.exists(fullpath):
                return fullpath

        msg = f'Could not find configuration file "{config_file}"!?'
        if verbose:
            print(msg)
            print("Tried the following directories:")
            for d in dirs:
                print(f" * {d}")

        raise FileNotFoundError(msg)  # type: ignore

    def get_data_file(self, filename: str) -> str:
        """
        Return the path to a data file.

        Parameters
        ----------
        filename: str
            Name of the data file

        Returns
        -------
        str
            Path to the data file
        """
        # If filename is an absolute path `os.path.join()` ignores
        # `self.data_dir`.
        if not filename:
            raise Exception('Argument "filename" should be provided!')
        return os.path.join(self.data_dir, filename)

    def set_folders(
        self, instance_type: InstanceType, instance_name: str, system_folders: bool
    ) -> None:
        """
        Set the folders where the configuration, data and log files are stored.

        Parameters
        ----------
        instance_type: InstanceType
            Type of instance that is checked
        instance_name: str
            Name of the configuration
        system_folders: bool
            Whether to use system folders rather than user folders
        """
        dirs = self.instance_folders(instance_type, instance_name, system_folders)

        # Check if the user has set custom directories
        custom_dirs = self.config.get("directories", None)
        if custom_dirs:
            log_dir = custom_dirs.get("log")
            data_dir = custom_dirs.get("data")

            self.log_dir = Path(log_dir) if log_dir else dirs.get("log")
            self.data_dir = Path(data_dir) if data_dir else dirs.get("data")
        else:
            self.log_dir = dirs.get("log")
            self.data_dir = dirs.get("data")

        # config dir could be different if the --config option is used
        self.config_dir = self.config_file.parent

    def setup_logging(self) -> None:
        """
        Setup a basic logging mechanism.

        Exits if the log file can't be created.
        """
        # TODO BvB 2023-03-31: would be nice to refactor this with the other
        # loggers in vantage6.common.log
        log_config = self.config["logging"]

        format_ = self.LOGGER_PREFIX + log_config["format"]
        datefmt = log_config.get("datefmt", "")

        # make sure the log-file exists
        os.makedirs(os.path.dirname(self.log_file), exist_ok=True)

        # Create the root logger
        logger, level = self.configure_logger(None, log_config["level"])

        # Create RotatingFileHandler
        try:
            rfh = logging.handlers.RotatingFileHandler(
                self.log_file,
                maxBytes=1024 * log_config["max_size"],
                backupCount=log_config["backup_count"],
            )
        except PermissionError:
            error(
                f"Can't write to log dir: {Fore.RED}{self.log_file}{Style.RESET_ALL}!"
            )
            exit(1)

        rfh.setLevel(level)
        rfh.setFormatter(logging.Formatter(format_, datefmt))
        logger.addHandler(rfh)

        # Check what to do with the console output ...
        if log_config["use_console"]:
            ch = ColorStreamHandler(sys.stdout)
            ch.setLevel(level)
            ch.setFormatter(logging.Formatter(format_, datefmt))
            logger.addHandler(ch)

        # control individual loggers
        loggers = log_config.get("loggers", [])
        for logger in loggers:
            self.configure_logger(logger["name"], logger["level"])

        # Finally, capture all warnings using the logging mechanism.
        logging.captureWarnings(True)

    @staticmethod
    def configure_logger(name: str | None, level: str) -> Tuple[logging.Logger, int]:
        """
        Set the logging level of a logger.

        Parameters
        ----------
        name : str
            Name of the logger to configure. If `None`, the root logger is
            configured.
        level : str
            Logging level to set. Must be one of 'debug', 'info', 'warning',
            'error', 'critical'.

        Returns
        -------
        Tuple[Logger, int]
            The logger object and the logging level that was set.
        """
        logger = logging.getLogger(name)
        level_ = getattr(logging, level.upper())
        logger.setLevel(level_)
        return logger, level



import uuid


def is_uuid(value: str) -> bool:
    """
    Check if value is a valid UUID.

     Parameters
    ----------
    value : str
        The value to check.

     Returns
    -------
    `True` if value is a valid UUID, otherwise `False`.
    """
    try:
        uuid_obj = uuid.UUID(value)
    except (ValueError, TypeError, AttributeError):
        return False
    return str(uuid_obj) == value



import hashlib
import json
import logging
import pathlib
import re
from http import HTTPStatus

import requests
from docker.auth import resolve_repository_name
from docker.utils import parse_repository_tag
from requests.auth import HTTPBasicAuth

from vantage6.common import logger_name

log = logging.getLogger(logger_name(__name__))


# TODO v5+ remove these dummy functions - just to prevent import issues from v4 CLI for
# the time being
def check_docker_running() -> bool:
    return True


def pull_image(image: str) -> None:
    pass


def get_container() -> str:
    return ""


def remove_container() -> None:
    pass


def remove_container_if_exists() -> None:
    pass


def get_server_config_name() -> str:
    return ""


def get_num_nonempty_networks() -> int:
    return 0


def get_network() -> str:
    return None


def delete_network() -> None:
    pass


def delete_volume_if_exists() -> None:
    pass


def stop_container() -> None:
    pass


def running_in_docker() -> bool:
    """
    Check if this code is executed within a Docker container.

    Returns
    -------
    bool
        True if the code is executed within a Docker container, False otherwise
    """
    return pathlib.Path("/.dockerenv").exists()


def parse_image_name(image: str) -> tuple[str, str, str]:
    """
    Parse image name into registry, repository, tag.

    The returned tag may also be a digest. If image contains both a tag and a digest,
    the tag will be returned rather than the digest.

    Parameters
    ----------
    image: str
        Image name. E.g. "harbor2.vantage6.ai/algorithms/average:latest" or
        "library/hello-world"

    Returns
    -------
    tuple[str, str, str]
        Registry, repository, and tag. Tag is "latest" if not specified in 'image'
    """
    registry_repository, tag = parse_repository_tag(image)
    tag = tag or "latest"
    if tag.startswith("sha256:"):
        # If the tag is a digest, the repository may include another tag, e.g. if
        # the image is "some-image:test@sha256:1234", the registry_repository would
        # still include the tag "test". If this is the case, set that as the tag,
        # because the tag is more reliable for policies than the digest (see
        # e.g. https://github.com/vantage6/vantage6/pull/1318#discussion_r1685560071)
        registry_repository, tag_ = parse_repository_tag(registry_repository)
        if tag_:
            tag = tag_
    registry, repository = resolve_repository_name(registry_repository)
    return registry, repository, tag


def get_image_name_wo_tag(image: str) -> str:
    """
    Get image name without tag

    Parameters
    ----------
    image: str
        Image name. E.g. "harbor2.vantage6.ai/algorithms/average:latest"

    Returns
    -------
    str
        Image name without tag. E.g. "harbor2.vantage6.ai/algorithms/average"
    """
    registry, repository, _ = parse_image_name(image)
    if registry == "docker.io":
        return repository
    else:
        return f"{registry}/{repository}"


def _get_manifest(
    full_image: str,
    registry_user: str | None = None,
    registry_password: str | None = None,
) -> requests.Response:
    """
    Get the manifest of an image

    This uses the OCI distribution specification which is supported by all major
    container registries.

    Parameters
    ----------
    full_image: str
        Image name. E.g. "harbor2.vantage6.ai/algorithms/average:latest"
    registry_user: str | None
        Docker username to authenticate with at the registry. Required if the image is
        private
    registry_password: str | None
        Docker password to authenticate with at the registry. Required if the image is
        private

    Returns
    -------
    requests.Response
        Response containing the manifest of the image

    Raises
    ------
    ValueError
        If the image name is invalid
    """
    registry, image, tag = parse_image_name(full_image)

    # if requesting from docker hub, manifests are at 'registry-1.docker.io'
    if registry == "docker.io":
        registry = "registry-1.docker.io"

    # request manifest. First try without authentication, as that is the most common
    # case. If that fails, try with authentication
    headers = {"Accept": "application/vnd.docker.distribution.manifest.v2+json"}
    manifest_endpoint = f"https://{registry}/v2/{image}/manifests/{tag}"
    response = requests.get(manifest_endpoint, headers=headers, timeout=60)

    # try requesting manifest using authentication from 'www-authenticate' header if
    # anonymous request failed. This has been tested with DockerHub and Github container
    # registry
    if (
        response.status_code == HTTPStatus.UNAUTHORIZED
        and "www-authenticate" in response.headers
    ):
        realm_pattern = r'Bearer realm="(?P<realm>[^"]+)"'
        scope_pattern = r'scope="(?P<scope>[^"]+)"'
        service_pattern = r'service="(?P<service>[^"]+)"'
        realm_match = re.match(realm_pattern, response.headers["www-authenticate"])
        scope_match = re.search(scope_pattern, response.headers["www-authenticate"])
        service_match = re.search(service_pattern, response.headers["www-authenticate"])
        if realm_match and scope_match and service_match:
            token_response = requests.get(
                realm_match.group("realm"),
                params={
                    "scope": scope_match.group("scope"),
                    "service": service_match.group("service"),
                },
                timeout=60,
            )
            if token_response.status_code == HTTPStatus.OK:
                token = token_response.json()["token"]
                response = requests.get(
                    manifest_endpoint,
                    headers={"Authorization": f"Bearer {token}", **headers},
                    timeout=60,
                )

    # If still getting unauthorize, try with username and password. The following code
    # was tested with private images on harbor2.vantage6.ai.
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response = requests.get(
            manifest_endpoint,
            headers=headers,
            auth=HTTPBasicAuth(registry_user, registry_password),
            timeout=60,
        )

    # handle errors or return manifest
    if response.status_code == HTTPStatus.NOT_FOUND:
        raise ValueError(f"Image {image}:{tag} from registry {registry} not found!")
    elif response.status_code != HTTPStatus.OK:
        raise ValueError(
            "Could not retrieve image manifest from "
            f"https://{registry}/v2/{image}/manifests/{tag}"
        )
    return response


def get_digest(
    full_image: str,
    registry_username: str | None = None,
    registry_password: str | None = None,
) -> str:
    """
    Get digest of an image by fetching the manifest

    Parameters
    ----------
    full_image: str
        Image name. E.g. "harbor2.vantage6.ai/algorithms/average:latest"
    registry_username: str | None
        Registry username to authenticate with at the registry. Required if the image is
        private
    registry_password: str | None
        Registry password to authenticate with at the registry. Required if the image is
        private

    Returns
    -------
    str
        Digest of the image
    """
    try:
        manifest_response = _get_manifest(
            full_image, registry_username, registry_password
        )
    except ValueError as exc:
        log.warning("Could not get manifest of image %s", full_image)
        log.warning("Error: %s", exc)
        return ""
    if "Docker-Content-Digest" in manifest_response.headers:
        return manifest_response.headers["Docker-Content-Digest"]
    else:
        return __calculate_digest(manifest_response.json())


def __calculate_digest(manifest: str) -> str:
    """
    Calculate the SHA256 digest from a Docker image manifest.
    Parameters
    ----------
    manifest : str
        Docker image manifest

    Returns
    -------
    str
        SHA256 digest of the manifest
    """
    # Serialize the manifest using canonical JSON
    serialized_manifest = json.dumps(
        manifest,
        indent=3,  # Believe it or not, this spacing is required to get the right SHA
    ).encode("utf-8")
    # Calculate the SHA256 digest
    digest = hashlib.sha256(serialized_manifest).hexdigest()
    return f"sha256:{digest}"



#!/usr/bin/env python3
"""
Test script to demonstrate the StrEnumBase functionality
"""

import unittest

from vantage6.common.enum import StrEnumBase


class TestEnum(StrEnumBase):
    """Test enum to demonstrate functionality"""

    VALUE_1 = "test1"
    VALUE_2 = "test2"
    VALUE_3 = "test3"


class TestEnumBase(unittest.TestCase):
    def test_enum_base_functionality(self):
        """Test the StrEnumBase list() method"""

        assert TestEnum.list() == ["test1", "test2", "test3"]
        assert TestEnum.names() == ["value_1", "value_2", "value_3"]
        assert TestEnum.items() == [
            ("value_1", "test1"),
            ("value_2", "test2"),
            ("value_3", "test3"),
        ]



from enum import IntEnum
from pathlib import Path

from vantage6.common.enum import StrEnumBase

#
#   PACKAGE GLOBALS
#
STRING_ENCODING = "utf-8"

APPNAME = "vantage6"

# TODO v5+ change to final v5 name
MAIN_VERSION_NAME = "5"

DEFAULT_DOCKER_REGISTRY = "harbor2.vantage6.ai"

DEFAULT_NODE_IMAGE = f"infrastructure/node:{MAIN_VERSION_NAME}"

DEFAULT_NODE_IMAGE_WO_TAG = "infrastructure/node"

DEFAULT_SERVER_IMAGE = f"infrastructure/server:{MAIN_VERSION_NAME}"

DEFAULT_UI_IMAGE = f"infrastructure/ui:{MAIN_VERSION_NAME}"

DEFAULT_ALGO_STORE_IMAGE = f"infrastructure/algorithm-store:{MAIN_VERSION_NAME}"

DEFAULT_ALPINE_IMAGE = "infrastructure/alpine:latest"

#  CHART GLOBALS
DEFAULT_CHART_REPO = "https://harbor2.vantage6.ai/chartrepo/infrastructure"

#
#   COMMON GLOBALS
#
PACKAGE_FOLDER = Path(__file__).parent.parent.parent

DATABASE_TYPES = ["csv", "parquet", "sql", "sparql", "excel", "other"]
FILE_BASED_DATABASE_TYPES = ["csv", "parquet", "excel", "other"]
SERVICE_BASED_DATABASE_TYPES = ["sql", "sparql", "other"]

PING_INTERVAL_SECONDS = 60

# Character to replace '=' with in encoded environment variables
ENV_VAR_EQUALS_REPLACEMENT = "!"

# default API path (for server and algorithm store)
DEFAULT_API_PATH = "/api"
DEFAULT_PROMETHEUS_EXPORTER_PORT = 7603

# Maximum interval to wait for requesting results from a task
MAX_INTERVAL = 300

# Constant multiplier to make interval for requesting results from a task progressively
# longer
INTERVAL_MULTIPLIER = 1.5

# keyword for the multiple dataframes in single argument decorator
DATAFRAME_MULTIPLE_KEYWORD = "multiple"
DATAFRAME_WITHIN_GROUP_SEPARATOR = ","
DATAFRAME_BETWEEN_GROUPS_SEPARATOR = ";"

# Default timeout for requests to the server
REQUEST_TIMEOUT = 300

# Default chunk size for streaming inputs and results
DEFAULT_CHUNK_SIZE = 1024 * 1024  # 1MB

# session state file name
SESSION_STATE_FILENAME = "session_state"

# session state file name
SESSION_STATE_FILENAME = "session_state"

# sandbox configuration suffix
SANDBOX_SUFFIX = ".sandbox"

# Localhost addresses
LOCALHOST = "localhost"
HTTP_LOCALHOST = "http://localhost"
LINUX_LOCALHOST = "http://127.0.0.1"


class InstanceType(StrEnumBase):
    """The types of instances that can be created."""

    NODE = "node"
    SERVER = "server"
    ALGORITHM_STORE = "algorithm-store"
    UI = "ui"
    AUTH = "auth"


class NodePolicy(StrEnumBase):
    """Enum containing the names of the names of the node policies"""

    ALLOWED_ALGORITHMS = "allowed_algorithms"
    ALLOWED_ALGORITHM_STORES = "allowed_algorithm_stores"
    ALLOWED_ORGANIZATIONS = "allowed_organizations"
    ALLOWED_USERS = "allowed_users"
    REQUIRE_ALGORITHM_PULL = "require_algorithm_pull"


class NodeConfigKey(StrEnumBase):
    """Enum containing the keys of the node configuration"""

    ENCRYPTION = "encryption"
    ALLOWED_ALGORITHMS = "allowed_algorithms"
    ALLOWED_ORGANIZATIONS = "allowed_orgs"
    ALLOWED_USERS = "allowed_users"
    DATABASE_LABELS = "database_labels"
    DATABASE_TYPES = "database_types"


class Ports(IntEnum):
    """Enum containing the default ports used by the vantage6 components"""

    HTTP = 80
    HTTPS = 443
    DEV_SERVER = 7601
    DEV_UI = 7600
    DEV_ALGO_STORE = 7602
    DEV_AUTH = 8080


class ContainerEnvNames(StrEnumBase):
    """Enum containing the names of the container environment variables"""

    FUNCTION_ACTION = "FUNCTION_ACTION"
    INPUT_FILE = "INPUT_FILE"
    ALGORITHM_METHOD = "ALGORITHM_METHOD"
    OUTPUT_FILE = "OUTPUT_FILE"
    SESSION_FOLDER = "SESSION_FOLDER"
    SESSION_FILE = "SESSION_FILE"
    HOST = "HOST"
    PORT = "PORT"
    API_PATH = "API_PATH"
    CONTAINER_TOKEN = "CONTAINER_TOKEN"
    DATABASE_URI = "DATABASE_URI"
    DATABASE_TYPE = "DATABASE_TYPE"
    DB_PARAM_PREFIX = "DB_PARAM_"
    USER_REQUESTED_DATAFRAMES = "USER_REQUESTED_DATAFRAMES"
    USER_REQUESTED_DATABASES = "USER_REQUESTED_DATABASES"
    TASK_ID = "TASK_ID"
    NODE_ID = "NODE_ID"
    ORGANIZATION_ID = "ORGANIZATION_ID"
    COLLABORATION_ID = "COLLABORATION_ID"


class RequiredNodeEnvVars(StrEnumBase):
    """Enum containing the required node environment variables"""

    V6_API_KEY = "V6_API_KEY"
    V6_NODE_NAME = "V6_NODE_NAME"
    KEYCLOAK_URL = "KEYCLOAK_URL"
    KEYCLOAK_REALM = "KEYCLOAK_REALM"


class AuthStatus(StrEnumBase):
    """Enum containing the different statuses of the authenticable (node/user)"""

    ONLINE = "online"
    OFFLINE = "offline"
